﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.IO;
using System.Resources;
using System.Diagnostics;
using System.Reflection;
using CsLEADUniversalViewer.Properties;
using Leadtools;
using Leadtools.Codecs;
using Leadtools.Demos;
using Leadtools.Controls;
using Leadtools.Multimedia;
using Leadtools.Drawing;
using Leadtools.Demos.Dialogs;

namespace CsLEADUniversalViewer
{
   public partial class MainForm : Form
   {
      RasterCodecs codecs = null;
      ImageViewerPanZoomInteractiveMode _panZoomInteractiveMode = null;
      int CurrentPageNum = 1;
      string _fileName = "";
      string _logFileName = "";
      string FolderName = "";
      //Flag to cancel the browsing process when the user presses Escape button
      bool CancelBrowse = false;
      //Recent Folders Queue
      Queue MRUlist = new Queue();
      //Load File Writer
      System.IO.StreamWriter LogFileWriter = null;
      //Loading resolution dialog
      LoadingResolutionDialog loadingResolutionDialog = null;
      //
      TableLayoutPanelCellPosition TempImageViewerCellInTableLayout;
      //
      Panel FullScreenPanel = new Panel();
      //
      FormBorderStyle MainFormBorderDefaultStyle;
      //
      Color ImageViewerDefaultBackColor;
      //
      private Leadtools.Multimedia.PlayCtrl playCtrl1;
      //
      bool PauseAnimation;
      //
      RasterImage animatedImage;
      //
      bool IsPlayingGif = false;

      public ImageViewerPanZoomInteractiveMode PanZoomInteractiveMode
      {
         get
         {
            return _panZoomInteractiveMode;
         }
         set
         {
            value = _panZoomInteractiveMode;
         }
      }

      public MainForm()
      {
         InitializeComponent();

         //Set LEADTOOLS license before deploying your application
         if (!Support.SetLicense())
            return;

         //RasterCodecs: provides support for loading and saving raster image files
         codecs = new RasterCodecs();

         _panZoomInteractiveMode = new ImageViewerPanZoomInteractiveMode();
         _panZoomInteractiveMode.AutoItemMode = ImageViewerAutoItemMode.AutoSet;
         _panZoomInteractiveMode.DoubleTapSizeMode = imageViewer1.SizeMode;
         _panZoomInteractiveMode.EnablePan = true;
         _panZoomInteractiveMode.EnablePinchZoom = false;
         _panZoomInteractiveMode.EnableZoom = false;

         tableLayoutPanel1.SetRowSpan(listBox1, 2);

         trkPosition.SetRange(0, 10000);

         loadingResolutionDialog = new LoadingResolutionDialog();

         TempImageViewerCellInTableLayout = new TableLayoutPanelCellPosition();

         FullScreenPanel.Visible = false;
      }

      private void MainForm_KeyDown(object sender, KeyEventArgs e)
      {
         if ((e.KeyCode == Keys.Escape))
         {
             if (CancelBrowse != true && FullScreenPanel.Visible != true)
            {
               CancelBrowse = true;
               //IsEscapePressed = true;
               MessageBox.Show("Browsing has been canceled");
            }

            ExitFullScreenMode(FullScreenPanel.Visible);
         }
      }

      private void MainForm_Load(object sender, EventArgs e)
      {
         try
         {
            this.playCtrl1 = new Leadtools.Multimedia.PlayCtrl();
            ((System.ComponentModel.ISupportInitialize)(this.playCtrl1)).BeginInit();
            // playCtrl1            
            this.playCtrl1.Dock = System.Windows.Forms.DockStyle.Fill;
            this.playCtrl1.Location = new System.Drawing.Point(0, 0);
            this.playCtrl1.Name = "playCtrl1";
            this.playCtrl1.OcxState = null;
            this.playCtrl1.Size = new System.Drawing.Size(1046, 713);
            this.playCtrl1.SourceFile = null;
            this.playCtrl1.TabIndex = 0;
            this.playCtrl1.Text = "playCtrl1";
            this.playCtrl1.DoubleClick += new System.EventHandler(this.playCtrl1_DoubleClick);
            this.playCtrl1.TrackingPositionChanged += new Leadtools.Multimedia.TrackingPositionChangedEventHandler(this.playCtrl1_TrackingPositionChanged);
            ((System.ComponentModel.ISupportInitialize)(this.playCtrl1)).EndInit();

            this.PlayerPanel.Controls.Add(this.playCtrl1);
            this.PlayerPanel.Controls.Add(this.imageViewer1);

            imageViewer1.Visible = playCtrl1.Visible = false;

            this.MaximumSize = this.Size;
            this.MinimumSize = new System.Drawing.Size(this.Size.Width - 70, this.Size.Height - 10);

            PauseAnimation = false;

            animatedImage = null;
         }
         catch (Exception)
         {
            DialogResult MsgBoxRes = MessageBox.Show(this, "LEADTOOLS Multimedia SDK is not installed. Please download and install it to continue using this demo. \n Do you want to download LEADTOOLS Multimedia SDK?", "Warning", MessageBoxButtons.YesNo, MessageBoxIcon.Warning);
            if (MsgBoxRes == System.Windows.Forms.DialogResult.Yes)
               System.Diagnostics.Process.Start("https://www.leadtools.com/downloads/default.htm?category=mm");
            else
               return;
         }

         this.KeyPreview = true;

         _logFileName = System.Environment.CurrentDirectory + "\\LeadUniversalViewerLog.txt";

         // First line in the Log file
         string strLogStart = "Start logging...\n\n";

         try
         {
            WriteToLogFile(_logFileName, strLogStart);

            //Load the text file that contains the names of the recent folders
            LoadRecentList();

            foreach (string item in MRUlist)
            {
               //populating menu
               ToolStripMenuItem fileRecent = new ToolStripMenuItem(item);
               //Add the files in the selcted folder
               fileRecent.MouseUp += new MouseEventHandler(fileRecent_MouseUp);
               //add the menu to "recent" menu
               recentToolStripMenuItem.DropDownItems.Add(fileRecent);
            }

            string FileDuration = string.Format("{0:D2}:{1:D2}:{2:D2}:{3:D3}",
                            0,
                            0,
                            0,
                            0);

            string FileCurrentPosition = string.Format("{0:D2}:{1:D2}:{2:D2}:{3:D3}",
                            0,
                            0,
                            0,
                            0);

            lbltrkPosition.Text = string.Format(" {0} / {1}", FileCurrentPosition, FileDuration);

            //Set the default loading resolutions
            codecs.Options.RasterizeDocument.Load.XResolution = 150;
            codecs.Options.RasterizeDocument.Load.YResolution = 150;

            InfoDialog UserInfoDialog = new InfoDialog();
            UserInfoDialog.Show(this);

            MainFormBorderDefaultStyle = this.FormBorderStyle;

            ImageViewerDefaultBackColor = imageViewer1.BackColor;
         }
         catch (Exception ex)
         {
            StackTrace st = new StackTrace();
            StackFrame sf = st.GetFrame(0);
            MethodBase currentMethodName = sf.GetMethod();

            WriteToLogFile(_logFileName, string.Format("{0} \t {1} \n", currentMethodName, ex.Message));
         }
      }

      private void ExitFullScreenMode(bool IsVisible)
      {
         if (IsVisible)
         {
            imageViewer1.Parent = PlayerPanel;
            tableLayoutPanel1.SetCellPosition(imageViewer1, TempImageViewerCellInTableLayout);

            FullScreenPanel.Dock = DockStyle.None;

            imageViewer1.Dock = DockStyle.Fill;

            imageViewer1.Visible = true;

            tableLayoutPanel1.Show();

            menuStrip1.Show();

            imageViewer1.BackColor = ImageViewerDefaultBackColor;

            this.FormBorderStyle = MainFormBorderDefaultStyle;

            FullScreenPanel.Visible = false;
         }
      }
      void BtnExitFullScreenMode_MouseClick(object sender, MouseEventArgs e)
      {
         ExitFullScreenMode(FullScreenPanel.Visible);
      }

      private void trkPosition_Scroll(object sender, System.EventArgs e)
      {
         try
         {
            playCtrl1.CurrentTrackingPosition = trkPosition.Value;

            SetSeekingButtonsState();

            TimeSpan t = TimeSpan.FromSeconds(playCtrl1.Duration);

            string FileDuration = string.Format("{0:D2}:{1:D2}:{2:D2}:{3:D3}",
                            t.Hours,
                            t.Minutes,
                            t.Seconds,
                            t.Milliseconds);

            t = TimeSpan.FromSeconds(playCtrl1.CurrentPosition);

            string FileCurrentPosition = string.Format("{0:D2}:{1:D2}:{2:D2}:{3:D3}",
                            t.Hours,
                            t.Minutes,
                            t.Seconds,
                            t.Milliseconds);

            lbltrkPosition.Text = string.Format(" {0} / {1}", FileCurrentPosition, FileDuration);

            lbltrkPosition.Refresh();
            trkPosition.Refresh();
         }
         catch (Exception ex)
         {
            StackTrace st = new StackTrace();
            StackFrame sf = st.GetFrame(0);
            MethodBase currentMethodName = sf.GetMethod();

            WriteToLogFile(_logFileName, string.Format("{0} \t {1} \n", currentMethodName, ex.Message));
         }
      }

      private string FormatPosition(double ts)
      {
         string h = null;
         string m = null;
         string s = null;
         h = ((int)(ts / 3600)).ToString();
         ts = ts % 3600;
         m = ((int)(ts / 60)).ToString();
         ts = ts % 60;
         s = ts.ToString("00.000");
         if (h.Length < 2)
         {
            h = "0" + h;
         }
         if (m.Length < 2)
         {
            m = "0" + m;
         }
         if (s.Length < 2)
         {
            s = "0" + s;
         }
         return h + ":" + m + ":" + s;
      }

      private void SetSeekingButtonsState()
      {
         PlaySeeking caps = 0;

         caps = playCtrl1.CheckSeekingCapabilities(PlaySeeking.Forward | PlaySeeking.Backward | PlaySeeking.FrameForward | PlaySeeking.FrameBackward);
         trkPosition.Enabled = (caps != 0);
      }

      private void ChangePlayerSizeMode()
      {
         //Toggle between normal and fit size modes of the player control
         if (playCtrl1.VideoWindowSizeMode == Leadtools.Multimedia.SizeMode.Fit)
         {
            playCtrl1.VideoWindowSizeMode = Leadtools.Multimedia.SizeMode.Normal;
            imageViewer1.Cursor = Cursors.Hand;
         }
         else
         {
            playCtrl1.VideoWindowSizeMode = Leadtools.Multimedia.SizeMode.Fit;
            imageViewer1.Cursor = Cursors.Default;
         }
      }

      private void ChangeViewerSizeMode()
      {
         //Toggle between ActualSize and fit size modes of the imageViewer control
         if (imageViewer1.SizeMode == ControlSizeMode.Fit)
         {
            imageViewer1.Zoom(ControlSizeMode.ActualSize, 1, imageViewer1.DefaultZoomOrigin);
            imageViewer1.Cursor = Cursors.Hand;
         }
         else
         {
            imageViewer1.Zoom(ControlSizeMode.Fit, 1, imageViewer1.DefaultZoomOrigin);
            imageViewer1.Cursor = Cursors.Default;
         }
      }

      private void imageViewer1_MouseDoubleClick(object sender, MouseEventArgs e)
      {
         //change the imageViewer's size mode when you double click the viewer
         try
         {
            if (imageViewer1.Image != null)
               ChangeViewerSizeMode();

            imageViewer1.Refresh();
         }
         catch (Exception ex)
         {
            StackTrace st = new StackTrace();
            StackFrame sf = st.GetFrame(0);
            MethodBase currentMethodName = sf.GetMethod();

            WriteToLogFile(_logFileName, string.Format("{0} \t {1} \n", currentMethodName, ex.Message));
         }
      }

      void fileRecent_MouseUp(object sender, MouseEventArgs e)
      {
         string currentFolder = sender.ToString();
         ToolStripMenuItem currentToolStripMenu = sender as ToolStripMenuItem;

         toolTip1.IsBalloon = false;

         if (Directory.Exists(currentFolder))
         {
            //Change the application's Cursor
            Cursor TempCursor = this.Cursor;
            this.Cursor = Cursors.WaitCursor;

            foreach (ToolStripMenuItem tempStripMenu in recentToolStripMenuItem.DropDownItems)
               tempStripMenu.Checked = false;

            currentToolStripMenu.Checked = true;

            //Flag to cancel the browsing process when the user presses Escape button
            CancelBrowse = false;

            menuStrip1.Enabled = false;

            AddFileToList(currentFolder);

            CancelBrowse = true;

            menuStrip1.Enabled = true;

            this.Cursor = TempCursor;
         }
      }

      private void WriteToLogFile(string LogFileName, string TextToWrite)
      {
         if (LogFileWriter == null)
         {
            LogFileWriter = new System.IO.StreamWriter(_logFileName);
            LogFileWriter.AutoFlush = true;
         }
         else if (System.IO.File.Exists(LogFileName))
            LogFileWriter = new System.IO.StreamWriter(LogFileName, true);

         LogFileWriter.WriteLine(TextToWrite);
         LogFileWriter.Close();
      }

      private void playMoreFilesToolStripMenuItem_Click(object sender, EventArgs e)
      {
         try
         {
            //Browse the folder that contains your images and media files
            using (FolderBrowserDialog BrowserDialog = new FolderBrowserDialog())
            {

               if (BrowserDialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
               {
                  FolderName = BrowserDialog.SelectedPath;

                  //Change the application's Cursor
                  Cursor TempCursor = this.Cursor;
                  this.Cursor = Cursors.WaitCursor;

                  toolTip1.IsBalloon = false;

                  //Flag to cancel the browsing process when the user presses Escape button                  
                  CancelBrowse = false;

                  menuStrip1.Enabled = false;

                  AddFileToList(FolderName);

                  CancelBrowse = true;

                  menuStrip1.Enabled = true;

                  ToolStripMenuItem fileRecent = new ToolStripMenuItem(FolderName);


                  if (!(MRUlist.Contains(fileRecent.Text))) //prevent duplication on recent list
                  {
                     while (MRUlist.Count >= 5)
                     {
                        MRUlist.Dequeue();
                     }

                     MRUlist.Enqueue(FolderName);

                     recentToolStripMenuItem.DropDownItems.Clear();

                     foreach (string item in MRUlist)
                     {
                        ToolStripMenuItem ItemToAdd = new ToolStripMenuItem(item);
                        ItemToAdd.MouseUp += new MouseEventHandler(fileRecent_MouseUp);
                        recentToolStripMenuItem.DropDownItems.Add(ItemToAdd);
                     }

                     menuStrip1.Refresh();
                  }

                  //Reset the application's Cursor to the default
                  this.Cursor = TempCursor;
               }
            }
         }
         catch (Exception ex)
         {
            StackTrace st = new StackTrace();
            StackFrame sf = st.GetFrame(0);
            MethodBase currentMethodName = sf.GetMethod();

            WriteToLogFile(_logFileName, string.Format("{0} \t {1} \n", currentMethodName, ex.Message));
         }
      }

      private void AddRasterImageToList(string fileName)
      {
         //try loading the current file
         using (RasterImage TempImage = codecs.Load(fileName, 1))
         {
            string strShortname = _fileName.Substring(fileName.LastIndexOf("\\"));
            strShortname = strShortname.TrimStart(("\\").ToCharArray());
            //Add the current file to the list-box
            MyListItem ItemToAdd = new MyListItem(fileName, strShortname, 1);//1 - Image File
            listBox1.Items.Add(ItemToAdd);
            Invoke((MethodInvoker)delegate { Application.DoEvents(); });
         }
      }

      //Add the valid file names to the list-box
      private void AddFileToList(string SelectedFolder)
      {
         try
         {
            //Get all files in the selected folder
            string[] filePaths = Directory.GetFiles(SelectedFolder);

            if (filePaths.Length == 0)
            {
               MessageBox.Show(string.Format("{0} <{1}>", "No files found in", SelectedFolder));
               return;
            }

            listBox1.DisplayMember = "FileShortName";
            listBox1.ValueMember = "FileType";


            listBox1.Items.Clear();
            Thread.Sleep(500);
            listBox1.Invalidate(true);


            string FileDuration = string.Format("{0:D2}:{1:D2}:{2:D2}:{3:D3}",
                            0,
                            0,
                            0,
                            0);

            string FileCurrentPosition = string.Format("{0:D2}:{1:D2}:{2:D2}:{3:D3}",
                            0,
                            0,
                            0,
                            0);

            lbltrkPosition.Text = string.Format(" {0} / {1}", FileCurrentPosition, FileDuration);

            codecs.Options.Load.AllPages = false;
            codecs.Options.Txt.Load.Enabled = supportLoadingTEXTFilesToolStripMenuItem.Checked;

            //Loop throgh the files
            for (int index = 0; index < filePaths.Length; index++)
            {
               if (CancelBrowse)
               {
                  CancelBrowse = false;
                  return;
               }

               Invoke((MethodInvoker)delegate { Application.DoEvents(); });

               //MediaInfo: provides detailed information about the source media file
               using (Leadtools.Multimedia.MediaInfo mediaInfo = new Leadtools.Multimedia.MediaInfo())
               {
                  _fileName = filePaths[index];

                  mediaInfo.SourceFile = _fileName;

                  if ((mediaInfo.SourceFormat == Leadtools.Multimedia.SourceFormatType.Unknown) || (mediaInfo.SourceFormat == Leadtools.Multimedia.SourceFormatType.Still))
                  //if the current file is image of any non-media file
                  {
                     try
                     {
                        if (CancelBrowse)
                        {
                           CancelBrowse = false;
                           return;
                        }

                        mediaInfo.ResetSource();

                        AddRasterImageToList(_fileName);

                        Invoke((MethodInvoker)delegate { Application.DoEvents(); });
                     }
                     catch (Exception ex)
                     {
                        StackTrace st = new StackTrace();
                        StackFrame sf = st.GetFrame(0);
                        MethodBase currentMethodName = sf.GetMethod();

                        WriteToLogFile(_logFileName, string.Format("{0} \t {1} \t {2} \n", currentMethodName, _fileName, ex.Message));

                        continue;
                     }
                  }
                  else
                  {
                     try
                     {
                        if (mediaInfo.OutputStreams == 0)
                        {
                           mediaInfo.ResetSource();
                           AddRasterImageToList(_fileName);
                        }
                        else
                        {
                           //Declare a temporary player control, and pass it to the Leadtools.Multimedia.PlayCtrl coconstructor
                           using (Leadtools.Multimedia.PlayCtrl TempPlayer = new Leadtools.Multimedia.PlayCtrl(true))
                           {
                              if (CancelBrowse)
                              {
                                 CancelBrowse = false;
                                 return;
                              }

                              TempPlayer.AutoStart = false;
                              TempPlayer.SourceFile = _fileName;

                              string strShortname = _fileName.Substring(_fileName.LastIndexOf("\\"));
                              strShortname = strShortname.TrimStart(("\\").ToCharArray());

                              MyListItem ItemToAdd = new MyListItem(_fileName, strShortname, 2);//2 - Media File (Video or Audio)
                              listBox1.Items.Add(ItemToAdd);

                              TempPlayer.ResetSource();
                           }
                           Invoke((MethodInvoker)delegate { Application.DoEvents(); });
                        }
                     }
                     catch (Exception ex)
                     {
                        StackTrace st = new StackTrace();
                        StackFrame sf = st.GetFrame(0);
                        MethodBase currentMethodName = sf.GetMethod();

                        WriteToLogFile(_logFileName, string.Format("{0} \t {1} \t {2} \n", currentMethodName, _fileName, ex.Message));

                        continue;
                     }
                  }
               }

               if (CancelBrowse)
               {
                  CancelBrowse = false;
                  return;
               }
            }

            btnNext.Visible = true;
            btnNext.Enabled = true;

            btnPrev.Visible = true;
            btnPrev.Enabled = true;

         }
         catch (Exception ex)
         {
            StackTrace st = new StackTrace();
            StackFrame sf = st.GetFrame(0);
            MethodBase currentMethodName = sf.GetMethod();

            WriteToLogFile(_logFileName, string.Format("{0} \t {1} \n", currentMethodName, ex.Message));
         }
      }

      private void SaveRecentFile(string path)
      {
         try
         {
            //keep list number not exceeded the given value
            while (MRUlist.Count > 5)
            {
               MRUlist.Dequeue();
            }

            //create file called "Recent.txt" located on app folder
            string RectFileName = System.Environment.CurrentDirectory + "\\Recent.txt";

            using (StreamWriter stringToWrite = new StreamWriter(RectFileName))
            {

               foreach (string item in MRUlist)
               {
                  stringToWrite.WriteLine(item); //write list to stream
               }

               stringToWrite.Flush(); //write stream to file
               stringToWrite.Close(); //close the stream and reclaim memory
            }

            //clear all recent list from menu
            recentToolStripMenuItem.DropDownItems.Clear();
         }
         catch (Exception ex)
         {
            StackTrace st = new StackTrace();
            StackFrame sf = st.GetFrame(0);
            MethodBase currentMethodName = sf.GetMethod();

            WriteToLogFile(_logFileName, string.Format("{0} \t {1} \n", currentMethodName, ex.Message));
         }
      }

      private void LoadRecentList()
      {
         //try to load file. If file isn't found, do nothing
         try
         {
            string RectFileName = System.Environment.CurrentDirectory + "\\Recent.txt";

            StreamReader listToRead = null;

            if (File.Exists(RectFileName))
            {
               //read file stream
               listToRead = new StreamReader(System.Environment.CurrentDirectory + "\\Recent.txt");
            }
            else
               return;

            string line;

            if (MRUlist.Count > 0)
               MRUlist.Clear();

            while ((line = listToRead.ReadLine()) != null) //read each line until end of file
               MRUlist.Enqueue(line); //insert to list

            listToRead.Close(); //close the stream

            if (listToRead != null)
               listToRead.Dispose();

         }
         catch (Exception ex)
         {
            StackTrace st = new StackTrace();
            StackFrame sf = st.GetFrame(0);
            MethodBase currentMethodName = sf.GetMethod();

            WriteToLogFile(_logFileName, string.Format("{0} \t {1} \n", currentMethodName, ex.Message));
         }
      }

      private void ListBox1_SelectedValueChanged(object sender, EventArgs e)
      {
         try
         {
            if (listBox1.SelectedIndex != -1)
            {
               MyListItem SelectedListItem = listBox1.SelectedItem as MyListItem;

               if (SelectedListItem != null)
               {
                  //if the selected file is an image, load it and then display it in the viewer
                  if ((SelectedListItem.FileType == 1))
                  {
                     // set the codecs.Options.Load.AllPages property to True to support loading all pages of the image file
                     codecs.Options.Load.AllPages = loadAllPagesToolStripMenuItem.Checked;

                     //Add the panning feature to the image viewer control
                     imageViewer1.InteractiveModes.Add(PanZoomInteractiveMode);
                     PanZoomInteractiveMode.IsEnabled = true;

                     if (IsPlayingGif)
                     {
                        IsPlayingGif = false;
                        PauseAnimation = false;
                     }

                     Invoke((MethodInvoker)delegate { Application.DoEvents(); });

                     if (imageViewer1.HasImage)
                        imageViewer1.Image.Page = 1;

                     //Load the image to the Viewer
                     imageViewer1.Image = codecs.Load(SelectedListItem.FileFullName);


                     CurrentPageNum = 1;

                     imageViewer1.Image.Page = CurrentPageNum;

                     numericUpDown1.Maximum = imageViewer1.Image.PageCount;
                     numericUpDown1.Value = CurrentPageNum;

                     //Change Visible and Enabled properties for some button and controls
                     btnFwd.Visible = true;
                     btnRew.Visible = true;

                     numericUpDown1.Visible = true;

                     if (imageViewer1.Image.PageCount > 1)
                     {
                        numericUpDown1.Enabled = true;
                        btnFwd.Enabled = true;
                        btnRew.Enabled = true;
                     }
                     else
                     {
                        numericUpDown1.Enabled = false;
                        btnFwd.Enabled = false;
                        btnRew.Enabled = false;
                     }

                     btnPlay.Visible = false;
                     btnStop.Visible = false;
                     btnPause.Visible = false;
                     trkPosition.Visible = false;
                     lbltrkPosition.Visible = false;
                     //Show the imageViewer control
                     imageViewer1.Visible = true;
                     imageViewer1.Dock = DockStyle.Fill;
                     imageViewer1.Zoom(ControlSizeMode.Fit, 1, imageViewer1.DefaultZoomOrigin);

                     using (CodecsImageInfo ImageInfo = codecs.GetInformation(SelectedListItem.FileFullName, true))
                     {
                        btnPlay.Visible = (ImageInfo.Format == RasterImageFormat.Gif);
                     }
                     //Hide the player control
                     playCtrl1.Visible = false;
                     playCtrl1.Dock = DockStyle.None;
                     playCtrl1.ResetSource();
                     playCtrl1.Stop();
                  }
                  else if (SelectedListItem.FileType == 2)
                  {
                     //Hide the imageViewer control
                     imageViewer1.Visible = false;
                     imageViewer1.Image = null;
                     imageViewer1.Dock = DockStyle.None;

                     //Show the player control
                     playCtrl1.Location = new Point(0, 0);
                     playCtrl1.Dock = DockStyle.Fill;
                     playCtrl1.Visible = true;

                     Leadtools.Multimedia.PlaySeeking playSeeking = playCtrl1.CheckSeekingCapabilities(Leadtools.Multimedia.PlaySeeking.Forward | Leadtools.Multimedia.PlaySeeking.Backward);

                     if (playSeeking == (Leadtools.Multimedia.PlaySeeking.Backward | Leadtools.Multimedia.PlaySeeking.Forward))
                        playCtrl1.CurrentTrackingPosition = 0;


                     PlayerPanel.Visible = true;

                     playCtrl1.AutoStart = playbackMediaFilesToolStripMenuItem.Checked;
                     playCtrl1.SourceFile = SelectedListItem.FileFullName;

                     trkPosition.Visible = true;
                     lbltrkPosition.Visible = true;

                     TimeSpan t = TimeSpan.FromSeconds(playCtrl1.Duration);

                     string FileDuration = string.Format("{0:D2}:{1:D2}:{2:D2}:{3:D3}",
                                     t.Hours,
                                     t.Minutes,
                                     t.Seconds,
                                     t.Milliseconds);

                     t = TimeSpan.FromSeconds(0);

                     string FileCurrentPosition = string.Format("{0:D2}:{1:D2}:{2:D2}:{3:D3}",
                                     t.Hours,
                                     t.Minutes,
                                     t.Seconds,
                                     t.Milliseconds);

                     lbltrkPosition.Text = string.Format(" {0} / {1}", FileCurrentPosition, FileDuration);

                     if (playCtrl1.RenderedStreams == Leadtools.Multimedia.StreamFormatType.Audio)
                     {
                        //If the selected file is an Audio file, show the AudioFile image from resources in the image viewer
                        ResourceManager rm = Resources.ResourceManager;
                        Image AudioImage = (Bitmap)rm.GetObject("AudioFile");

                        imageViewer1.Image = Leadtools.Drawing.RasterImageConverter.ConvertFromImage(AudioImage, Leadtools.Drawing.ConvertFromImageOptions.None);
                        imageViewer1.Dock = DockStyle.Fill;
                        playCtrl1.Hide();
                        imageViewer1.Visible = true;
                        imageViewer1.Show();
                     }

                     //Change Visible and Enabled properties for some button and controls
                     numericUpDown1.Enabled = false;
                     btnFwd.Enabled = true;
                     btnRew.Enabled = true;

                     btnFwd.Visible = true;
                     btnRew.Visible = true;
                     numericUpDown1.Visible = false;
                     btnPlay.Visible = true;
                     btnStop.Visible = true;
                     btnPause.Visible = true;

                     playCtrl1.Refresh();

                     listBox1.Focus();
                  }
                  else
                  {
                     //If the selected file is not valid or not supported, show the NotSupportedFormat image from resources in the image viewer
                     ResourceManager rm = Resources.ResourceManager;
                     Image NotSupportedFormatImage = (Bitmap)rm.GetObject("NotSupportedFormat");

                     imageViewer1.Image = Leadtools.Drawing.RasterImageConverter.ConvertFromImage(NotSupportedFormatImage, Leadtools.Drawing.ConvertFromImageOptions.None);
                     imageViewer1.Dock = DockStyle.Fill;
                     playCtrl1.Hide();
                     imageViewer1.Visible = true;
                     imageViewer1.Show();
                  }
               }

               this.Refresh();

               imageViewer1.Cursor = Cursors.Default;
            }
         }
         catch (Exception ex)
         {
            StackTrace st = new StackTrace();
            StackFrame sf = st.GetFrame(0);
            MethodBase currentMethodName = sf.GetMethod();

            WriteToLogFile(_logFileName, string.Format("{0} \t {1} \n", currentMethodName, ex.Message));
         }
      }

      private void btnRew_Click(object sender, EventArgs e)
      {
         try
         {
            //Get the selected item from the list-box
            MyListItem SelectedItem = listBox1.SelectedItem as MyListItem;

            //If the selected file is multi-page file, show the Previous page
            if (SelectedItem.FileType == 1)
            {
               if (imageViewer1.Visible && imageViewer1.HasImage)
                  ShowPreviousPage();
            }
            //If the selected file is a media file, change the playback rate
            else if (SelectedItem.FileType == 2)
               if (playCtrl1.Rate > 0.5)
                  playCtrl1.Rate = playCtrl1.Rate / 2;
         }
         catch (Exception ex)
         {
            StackTrace st = new StackTrace();
            StackFrame sf = st.GetFrame(0);
            MethodBase currentMethodName = sf.GetMethod();

            WriteToLogFile(_logFileName, string.Format("{0} \t {1} \n", currentMethodName, ex.Message));
         }
      }

      private void btnFwd_Click(object sender, EventArgs e)
      {
         try
         {
            //Get the selected item from the list-box
            MyListItem SelectedItem = listBox1.SelectedItem as MyListItem;

            //If the selected file is multi-page file, show the next page
            if (SelectedItem.FileType == 1)
            {
               if (imageViewer1.Visible && imageViewer1.HasImage)
                  ShowNextPage();
            }

            //If the selected file is a media file, change the playback rate
            else if (SelectedItem.FileType == 2)
               if (playCtrl1.Rate < 2)
                  playCtrl1.Rate = playCtrl1.Rate * 2;
         }
         catch (Exception ex)
         {
            StackTrace st = new StackTrace();
            StackFrame sf = st.GetFrame(0);
            MethodBase currentMethodName = sf.GetMethod();

            WriteToLogFile(_logFileName, string.Format("{0} \t {1} \n", currentMethodName, ex.Message));
         }
      }

      private void btnNext_Click(object sender, EventArgs e)
      {
         //Show the next file in the list-box
         int NextIndex = -1;

         if (listBox1.Items.Count > 1)
         {
            MyListItem SelectedItem = listBox1.SelectedItem as MyListItem;

            if (SelectedItem != null)
            {
               if (listBox1.Items.IndexOf(SelectedItem) != (listBox1.Items.Count - 1))
               {
                  NextIndex = listBox1.Items.IndexOf(SelectedItem) + 1;

                  if (listBox1.Items[NextIndex] != null)
                     listBox1.SelectedIndex = NextIndex;
               }
            }
         }
      }

      //Show previous page in multi-page image
      private void ShowPreviousPage()
      {
         if (CurrentPageNum == 1)
            CurrentPageNum = 1;
         else
            CurrentPageNum--;

         imageViewer1.Image.Page = CurrentPageNum;

         numericUpDown1.Maximum = imageViewer1.Image.PageCount;

         numericUpDown1.Value = CurrentPageNum;

         imageViewer1.Refresh();
      }

      //Show next page in multi-page image
      private void ShowNextPage()
      {
         if (CurrentPageNum == imageViewer1.Image.PageCount)
            CurrentPageNum = imageViewer1.Image.PageCount;
         else
            CurrentPageNum++;

         imageViewer1.Image.Page = CurrentPageNum;

         numericUpDown1.Maximum = imageViewer1.Image.PageCount;

         numericUpDown1.Value = CurrentPageNum;

         imageViewer1.Refresh();
      }

      private void btnPrev_Click(object sender, EventArgs e)
      {
         //Show the next file in the list-box
         int PrevIndex = -1;

         if (listBox1.Items.Count > 1)
         {
            MyListItem SelectedItem = listBox1.SelectedItem as MyListItem;
            if (SelectedItem != null)
            {
               if (listBox1.Items.IndexOf(SelectedItem) != 0)
               {
                  PrevIndex = listBox1.Items.IndexOf(SelectedItem) - 1;

                  if (listBox1.Items[PrevIndex] != null)
                     listBox1.SelectedIndex = PrevIndex;
               }
            }
         }
      }

      //Change the current page using the numericUpDown control
      private void numericUpDown1_ValueChanged(object sender, EventArgs e)
      {
         if (((int)numericUpDown1.Value >= 1) && ((int)numericUpDown1.Value <= imageViewer1.Image.PageCount))
         {
            imageViewer1.Image.Page = (int)numericUpDown1.Value;
         }
      }

      //
      private void PlayGifFile(RasterImage GifImage)
      {
         try
         {
            if (IsPlayingGif)
            {
               int CurrentPage = 1;

               while (CurrentPage <= animatedImage.PageCount)
               {
                  //imageViewer1.Image.Page = CurrentPage;
                  numericUpDown1.Value = CurrentPage;

                  imageViewer1.Refresh();
                  Invoke((MethodInvoker)delegate { Application.DoEvents(); });

                  Thread.Sleep(200);

                  CurrentPage++;

                  if (PauseAnimation)
                  {
                     IsPlayingGif = false;
                     break;
                  }
               }
            }
         }
         catch (Exception ex)
         {
            StackTrace st = new StackTrace();
            StackFrame sf = st.GetFrame(0);
            MethodBase currentMethodName = sf.GetMethod();

            WriteToLogFile(_logFileName, string.Format("{0} \t {1} \n", currentMethodName, ex.Message));
         }
      }

      //Play button
      private void btnPlay_Click(object sender, EventArgs e)
      {
         MyListItem SelectedListItem = listBox1.SelectedItem as MyListItem;

         if (SelectedListItem != null)
         {
            if (SelectedListItem.FileType == 1)
            {
               if (SelectedListItem != null)
               {
                  using (CodecsImageInfo ImageInfo = codecs.GetInformation(SelectedListItem.FileFullName, true))
                  {
                     if (ImageInfo.Format == RasterImageFormat.Gif)
                     {
                        animatedImage = codecs.Load(SelectedListItem.FileFullName);
                        IsPlayingGif = true;
                        PlayGifFile(animatedImage);
                        IsPlayingGif = false;
                     }
                  }
               }
            }
            else if (SelectedListItem.FileType == 2)
               playCtrl1.Run();
         }
      }

      //Pause button
      private void btnPause_Click(object sender, EventArgs e)
      {
         playCtrl1.Pause();
      }
      //Stop button
      private void btnStop_Click(object sender, EventArgs e)
      {
         playCtrl1.Stop();
      }

      //Enable or disable loading all pages
      private void loadAllPagesToolStripMenuItem_Click(object sender, EventArgs e)
      {
         loadAllPagesToolStripMenuItem.Checked = !loadAllPagesToolStripMenuItem.Checked;

         MyListItem SelectedItem = listBox1.SelectedItem as MyListItem;

         if (SelectedItem != null)
         {
            listBox1.BeginUpdate();
            listBox1.SetSelected(listBox1.SelectedIndex, true);
            listBox1.EndUpdate();
         }
      }

      //Enable or disable Instant playback of selected media files
      private void playbackMediaFilesToolStripMenuItem_Click(object sender, EventArgs e)
      {
         playbackMediaFilesToolStripMenuItem.Checked = !playbackMediaFilesToolStripMenuItem.Checked;
      }

      //Change the player's control size mode when the use double clicks the control
      private void playCtrl1_DoubleClick(object sender, EventArgs e)
      {
         try
         {
            if (playCtrl1.SourceFile != null)
               ChangePlayerSizeMode();
         }
         catch (Exception ex)
         {
            StackTrace st = new StackTrace();
            StackFrame sf = st.GetFrame(0);
            MethodBase currentMethodName = sf.GetMethod();

            WriteToLogFile(_logFileName, string.Format("{0} \t {1} \n", currentMethodName, ex.Message));
         }
      }

      private void playCtrl1_TrackingPositionChanged(object sender, TrackingPositionChangedEventArgs e)
      {
         try
         {
            trkPosition.Value = e.position;
            SetSeekingButtonsState();

            TimeSpan t = TimeSpan.FromSeconds(playCtrl1.Duration);

            string FileDuration = string.Format("{0:D2}:{1:D2}:{2:D2}:{3:D3}",
                           t.Hours,
                           t.Minutes,
                           t.Seconds,
                           t.Milliseconds);

            t = TimeSpan.FromSeconds(playCtrl1.CurrentPosition);

            string FileCurrentPosition = string.Format("{0:D2}:{1:D2}:{2:D2}:{3:D3}",
                            t.Hours,
                            t.Minutes,
                            t.Seconds,
                            t.Milliseconds);

            lbltrkPosition.Text = string.Format(" {0} / {1}", FileCurrentPosition, FileDuration);

            lbltrkPosition.Refresh();
            trkPosition.Refresh();
         }
         catch (Exception ex)
         {
            StackTrace st = new StackTrace();
            StackFrame sf = st.GetFrame(0);
            MethodBase currentMethodName = sf.GetMethod();

            WriteToLogFile(_logFileName, string.Format("{0} \t {1} \n", currentMethodName, ex.Message));
         }
      }

      private void clearAllToolStripMenuItem_Click(object sender, EventArgs e)
      {
         recentToolStripMenuItem.DropDownItems.Clear();
      }

      private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
      {
         string RectFileName = System.Environment.CurrentDirectory + "\\Recent.txt";
         SaveRecentFile(RectFileName);
      }

      private void exitToolStripMenuItem_Click(object sender, EventArgs e)
      {
         Application.Exit();
      }

      private void listBox1_MouseMove(object sender, MouseEventArgs e)
      {
         Point point = listBox1.PointToClient(Cursor.Position);

         //Get the index of the current listbox item
         int index = listBox1.IndexFromPoint(point);

         if (index < 0) return;

         //Get the listbox item in the selected index
         MyListItem SelectedListItem = listBox1.Items[index] as MyListItem;

         //Get the current text of the toolTip
         string strTool = toolTip1.GetToolTip(listBox1);

         //If the text of the toolTip does not equal the text of the selected item, show the text of the item
         if (strTool != SelectedListItem.FileShortName)
         {
            if (SelectedListItem != null)
               toolTip1.SetToolTip(listBox1, SelectedListItem.FileShortName);
         }
      }

      private void changeloadingOrRasterizingResolutionToolStripMenuItem_Click(object sender, EventArgs e)
      {
         loadingResolutionDialog.ShowDialog();

         codecs.Options.RasterizeDocument.Load.XResolution = loadingResolutionDialog.LoadingResolution;
         codecs.Options.RasterizeDocument.Load.YResolution = loadingResolutionDialog.LoadingResolution;

         loadingResolutionDialog.Hide();
      }

      private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
      {
         using (AboutDialog aboutDialog = new AboutDialog("Universal Viewer", ProgrammingInterface.CS))
            aboutDialog.ShowDialog(this);
      }

      private void ShowViewerInFullScreen(bool IsVisible)
      {
         if (IsVisible == false)
         {
            TempImageViewerCellInTableLayout = tableLayoutPanel1.GetCellPosition(imageViewer1);

            FullScreenPanel.Width = this.Width;
            FullScreenPanel.Height = this.Height;
            FullScreenPanel.Dock = DockStyle.Fill;

            imageViewer1.Parent = FullScreenPanel;
            imageViewer1.Dock = DockStyle.Fill;

            this.WindowState = FormWindowState.Maximized;
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.Sizable;

            this.Controls.Add(FullScreenPanel);
            FullScreenPanel.BringToFront();
            tableLayoutPanel1.Hide();

            menuStrip1.Hide();

            imageViewer1.Zoom(ControlSizeMode.ActualSize, 1, imageViewer1.DefaultZoomOrigin);

            imageViewer1.BackColor = Color.Black;

            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.None;

            FullScreenPanel.Visible = true;

            this.Refresh();

            FullScreenPanel.Refresh();

            imageViewer1.Refresh(); 
         }
      }
      private void btnFullScreen_Click(object sender, EventArgs e)
      {
         if (imageViewer1.Visible)
            ShowViewerInFullScreen(FullScreenPanel.Visible);
         else if (playCtrl1.Visible)
            playCtrl1.ToggleFullScreenMode();
      }

      private void supportLoadingTEXTFilesToolStripMenuItem_Click(object sender, EventArgs e)
      {
         supportLoadingTEXTFilesToolStripMenuItem.Checked = !supportLoadingTEXTFilesToolStripMenuItem.Checked;
      }

      private void btnFwd_MouseHover(object sender, EventArgs e)
      {
         MyListItem SelectedItem = listBox1.SelectedItem as MyListItem;

         if (SelectedItem != null)
            if (SelectedItem.FileType == 1)
            {
               toolTip1.SetToolTip(btnFwd, "Next page");
            }
            else if (SelectedItem.FileType == 2)
            {
               toolTip1.SetToolTip(btnFwd, "Double Speed");
            }
      }

      private void btnRew_MouseHover(object sender, EventArgs e)
      {
         MyListItem SelectedItem = listBox1.SelectedItem as MyListItem;

         if (SelectedItem != null)
            if (SelectedItem.FileType == 1)
            {
               toolTip1.SetToolTip(btnRew, "Previous page");
            }
            else if (SelectedItem.FileType == 2)
            {
               toolTip1.SetToolTip(btnRew, "Half Speed");
            }
      }
   }
}